function [powerFunctionZ, settings, statisticDistribution, notFeasibleRatio, notFeasibleThrownOutRatio] = savePower( functionNames, underlyingProcess, VaRProcess, alphaVaR, alphaTest, lookback, dataLength, sims, powerV, powerPhi, powerP, id)
%SAVEPOWER Saves (to file) the power of the tests given the underlying process and
%VaR estimation procedure
%   functionNames: Cell of strings, e.g. {'testChristoffersen',
%   'testKupiec'}
%   underlyingProcess: String input to generateData. If 'uniform' it
%   generates data from the breach series
%   VaRProcess: String input to generateData ('normal' is the only option
%   so far)
%   alphaVaR: Table of VaR-levels, e.g. [0.01 0.05 0.1]
%   alphaTest: Double Test significance level
%   lookback: Integer
%   dataLength: Length of the return/VaR series
%   simulations: # simulations per point of the power function
%   powerV: Table of degrees of freedoms for the underlying process, e.g.
%   [3:1:10]
%   powerPhi: Table of garch parameters for the underlying process, e.g.
%   [0:0.1:1]
%   powerP: Table of alternative hypothesis for the uniform distribution
%   (breach probability) e.g. [0:0.01:1]
%   id: Integer to set some identification at the end of the filename


if(strcmp(underlyingProcess, 'uniform'))
    powerV = [0];
    powerPhi = [0];
elseif(strcmp(underlyingProcess, 'normal'))
    powerV = [0];
    powerPhi = [0];
    powerP = [0];
elseif(strcmp(underlyingProcess, 'ngarchstudentt'))
    powerP = [0];
    garchParameters = powerPhi;
    powerPhi = [0];
elseif(strcmp(underlyingProcess, 'studentt'))
    powerP = [0];
    powerPhi = [0];
elseif(strcmp(underlyingProcess, 'garchnormal'))   
    powerP = [0];
    powerV = [0];
elseif(strcmp(underlyingProcess, 'garchstudentt'))
    powerP = [0];
else
    error(['Unknown underlying process; ', underlyingProcess]);
end

if(strcmp(underlyingProcess, 'ngarchstudentt'))
    
    powerFunction = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi));
    notFeasibleRatio = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi));
    notFeasibleThrownOutRatio = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi));
    
    statisticDistribution = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi), sims);

    settings = struct('functionNames', functionNames, 'alphaVaR', alphaVaR, 'powerV', powerV, 'garchParameters', garchParameters, 'powerP', powerP, 'underlyingProcess', underlyingProcess, 'VaRProcess', VaRProcess, 'alphaTest', alphaTest, 'dataLength', dataLength, 'lookback', lookback, 'sims', sims);

    for i=1:length(alphaVaR)
        for j=1:length(functionNames)

                functionName = functionNames{j};
                distribution = getDistribution(functionName, alphaVaR(i), dataLength, 50000, 0);

                for k=1:length(powerP)
                    for l=1:length(powerV)
                        for m=1:length(powerPhi)
                            [powerFunction(i, j, k, l, m), statisticDistribution(i, j, k, l, m, :), notFeasibleRatio(i, j, k, l, m), notFeasibleThrownOutRatio(i, j, k, l, m)]  = getPow(functionName, underlyingProcess, VaRProcess, alphaVaR(i), powerV(l), garchParameters, powerP(k), dataLength, alphaTest, distribution, lookback, sims);
                        end
                    end
                end

                disp([functionName, '_', underlyingProcess, '_', VaRProcess, '_', num2str(alphaVaR(i)), '_', num2str(lookback), '_', num2str(dataLength), '_', num2str(alphaTest), '_', mat2str(garchParameters), '_', num2str(id)]);
        end
    end

    powerFunctionZ = powerFunction;
else

    powerFunction = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi));
    notFeasibleRatio = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi));
    notFeasibleThrownOutRatio = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi));
                                
    statisticDistribution = zeros( length(alphaVaR), length(functionNames), length(powerP), length(powerV), length(powerPhi), sims);


    settings = struct('functionNames', functionNames, 'alphaVaR', alphaVaR, 'powerV', powerV, 'powerPhi', powerPhi, 'powerP', powerP, 'underlyingProcess', underlyingProcess, 'VaRProcess', VaRProcess, 'alphaTest', alphaTest, 'dataLength', dataLength, 'lookback', lookback, 'sims', sims);

    for i=1:length(alphaVaR)
        for j=1:length(functionNames)

                functionName = functionNames{j};
                distribution = getDistribution(functionName, alphaVaR(i), dataLength, 50000, 0);

                for k=1:length(powerP)
                    for l=1:length(powerV)
                        for m=1:length(powerPhi)
                            [powerFunction(i, j, k, l, m), statisticDistribution(i, j, k, l, m, :), notFeasibleRatio(i, j, k, l, m), notFeasibleThrownOutRatio(i, j, k, l, m)] = getPow(functionName, underlyingProcess, VaRProcess, alphaVaR(i), powerV(l), powerPhi(m), powerP(k), dataLength, alphaTest, distribution, lookback, sims);
                        end
                    end
                end

                %alpha = alphaVaR(i);
                %savefile = [savePath, functionName, '_', underlyingProcess, '_', VaRProcess, '_', num2str(alphaVaR(i)), '_', num2str(lookback), '_', num2str(dataLength), '_', num2str(alphaTest), '_', num2str(id), '.mat'];
                %save(savefile, 'powerFunctionZ', 'powerV', 'powerPhi', 'powerP', 'functionName', 'underlyingProcess', 'VaRProcess', 'alpha', 'alphaTest', 'lookback', 'dataLength', 'sims');
                disp([functionName, '_', underlyingProcess, '_', VaRProcess, '_', num2str(alphaVaR(i)), '_', num2str(lookback), '_', num2str(dataLength), '_', num2str(alphaTest), '_', num2str(id)]);
        end
    end

    powerFunctionZ = powerFunction;
end
end

